/*
 * table.h -- symbol table
 */


#ifndef _TABLE_H_
#define _TABLE_H_


#define ENTRY_KIND_TYPE		0
#define ENTRY_KIND_VAR		1
#define ENTRY_KIND_PROC		2


typedef struct {
  int kind;
  union {
    struct {
      Type *type;
    } typeEntry;
    struct {
      Type *type;
      boolean isRef;
    } varEntry;
    struct {
      ParamTypes *paramTypes;
      struct table *localTable;
    } procEntry;
  } u;
} Entry;

/** stores the records of a symbol table in a binary tree data structure
 *
 * In a binary tree, records are ordered by the size of a unique number associated with 
 * each of the nodes. The node with the least value is stored at the bottom left, the node 
 * with the highest values is stored at the bottom right. That way, node lookups are pretty 
 * fast.
 * 
 * In our case, the "key" entry represents this unique order number; it is the same as the 
 * "unique random stamp" of each symbol.
 */
typedef struct bintree {
  Symbol *symbol;
  unsigned key;
  Entry *entry;
  struct bintree *left;
  struct bintree *right;
} Bintree;


typedef struct table {
  Bintree *bintree; /* contains the tables' data, i.e. effectively _is_ the symbol table */
  struct table *upperLevel; /* points to the Table one level above, i.e. one level more global */
} Table;


Entry *newTypeEntry(Type *type);
Entry *newVarEntry(Type *type, boolean isRef);
Entry *newProcEntry(ParamTypes *paramTypes, Table *localTable);

Table *newTable(Table *upperLevel);
Entry *enter(Table *table, Symbol *symbol, Entry *entry);
Entry *lookup(Table *table, Symbol *symbol);

void showEntry(Entry *entry);
void showTable(Table *table);


#endif /* _TABLE_H_ */
